import uuid
from abc import abstractmethod
from typing import List

from sziszapangma.core.alignment.alignment_classic_calculator import AlignmentClassicCalculator
from sziszapangma.core.wer.wer_calculator import WerCalculator
from sziszapangma.integration.mapper.alignment_step_mapper import AlignmentStepMapper
from sziszapangma.integration.repository.experiment_repository import ExperimentRepository
from sziszapangma.integration.task.processing_task import ProcessingTask
from sziszapangma.model.model import Word
from sziszapangma.model.relation_manager import RelationManager


class SentenceWerProcessor(ProcessingTask):
    _gold_transcript_property_name: str
    _asr_property_name: str
    _processing_alignment_property_name: str
    _processing_metric_property_name: str
    _alignment_classic_calculator: AlignmentClassicCalculator
    _wer_calculator: WerCalculator

    def __init__(self, gold_transcript_property_name: str, asr_property_name: str,
                 processing_alignment_property_name: str, processing_metric_property_name: str, task_name: str,
                 require_update: bool):
        super().__init__(task_name, require_update)
        self._gold_transcript_property_name = gold_transcript_property_name
        self._asr_property_name = asr_property_name
        self._processing_metric_property_name = processing_metric_property_name
        self._processing_alignment_property_name = processing_alignment_property_name
        self._alignment_classic_calculator = AlignmentClassicCalculator()
        self._wer_calculator = WerCalculator()

    def get_gold_transcript_text(self, record_id: str, experiment_repository: ExperimentRepository) -> str:
        property_value = experiment_repository.get_property_for_key(record_id, self._gold_transcript_property_name)
        return property_value['gold_transcript_raw']

    def get_asr_text(self, record_id: str, experiment_repository: ExperimentRepository) -> str:
        property_value = experiment_repository.get_property_for_key(record_id, self._asr_property_name)
        return property_value['full_text']

    def run_single_process(self, record_id: str, experiment_repository: ExperimentRepository):
        gold_transcript_text = self.get_gold_transcript_text(record_id, experiment_repository)
        asr_text = self.get_asr_text(record_id, experiment_repository)
        alignment = self._alignment_classic_calculator.calculate_alignment(
            reference=self.get_words_from_text_list(self.process_text(gold_transcript_text)),
            hypothesis=self.get_words_from_text_list(self.process_text(asr_text)),
        )
        wer = self._wer_calculator.calculate_wer(alignment)
        values_to_save = {
            self._processing_alignment_property_name: [AlignmentStepMapper.to_json_dict(it) for it in alignment],
            self._processing_metric_property_name: wer}
        for key in values_to_save.keys():
            experiment_repository.update_property_for_key(record_id, key, values_to_save[key])

    def skip_for_record(self, record_id: str, experiment_repository: ExperimentRepository) -> bool:
        properties = [self._processing_alignment_property_name, self._processing_metric_property_name]
        return all([experiment_repository.get_property_for_key(record_id, it) is not None for it in properties])

    @abstractmethod
    def process_text(self, text: str) -> List[str]:
        pass

    @staticmethod
    def get_words_from_text_list(values: List[str]) -> List[Word]:
        return [Word(id=str(uuid.uuid4()), type='Word', text=it) for it in values]
